"""The version number of pyjob is recorded here"""

# Maintain sematantic versioning. Further information can
# be found here [http://semver.org/]
__version_info__ = (0, 1, 2)

# ======================================================
# Do __NOT__ change anything below here
__version__ = '.'.join(str(v) for v in __version_info__)
